"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const cli_1 = require("@yarnpkg/cli");
const core_1 = require("@yarnpkg/core");
const clipanion_1 = require("clipanion");
const Constraints_1 = require("../../Constraints");
// eslint-disable-next-line arca/no-default-export
class ConstraintsSourceCommand extends cli_1.BaseCommand {
    constructor() {
        super(...arguments);
        this.verbose = clipanion_1.Option.Boolean(`-v,--verbose`, false, {
            description: `Also print the fact database automatically compiled from the workspace manifests`,
        });
    }
    async execute() {
        const configuration = await core_1.Configuration.find(this.context.cwd, this.context.plugins);
        const { project } = await core_1.Project.find(configuration, this.context.cwd);
        const constraints = await Constraints_1.Constraints.find(project);
        this.context.stdout.write(this.verbose ? constraints.fullSource : constraints.source);
    }
}
exports.default = ConstraintsSourceCommand;
ConstraintsSourceCommand.paths = [
    [`constraints`, `source`],
];
ConstraintsSourceCommand.usage = clipanion_1.Command.Usage({
    category: `Constraints-related commands`,
    description: `print the source code for the constraints`,
    details: `
      This command will print the Prolog source code used by the constraints engine. Adding the \`-v,--verbose\` flag will print the *full* source code, including the fact database automatically compiled from the workspace manifests.
    `,
    examples: [[
            `Prints the source code`,
            `yarn constraints source`,
        ], [
            `Print the source code and the fact database`,
            `yarn constraints source -v`,
        ]],
});
